/*
 * tools/lib/lvm_dir_cache.c
 *
 * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
 *
 * May 1998
 * January 1999
 *
 *
 * This LVM library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This LVM library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this LVM library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA
 *
 */

/*
 * Changelog
 *
 *   24/01/1999 - corrected to new devfs name space
 *
 */

#include <liblvm.h>

char *dirname = NULL;
int lvm_dir_cache_select ( struct dirent*);
int lvm_dir_cache_hit ( dir_cache_t*, int, dev_t);


int lvm_dir_cache ( dir_cache_t **dir_cache_ptr) {
   static int cache_size = 0;
   int d = 0;
   int dirent_count = 0;
   int n = 0;
   char name[NAME_LEN] = { 0, };
   static char *devdir[] = {
      "/dev/ide/hd",
      "/dev/sd",
      "/dev/md",
      "/dev",
      NULL
   };
   struct dirent **dirent = NULL;
   struct stat stat_b;
   static dir_cache_t *dir_cache = NULL;
   dir_cache_t *dir_cache_sav = NULL;


#ifdef DEBUG
         debug ( "lvm_dir_cache -- CALLED\n");
#endif
   if ( dir_cache_ptr == NULL) return -LVM_EPARAM;

   if ( dir_cache == NULL) {
      for ( d = 0; devdir[d] != NULL; d++) {
         dirname = devdir[d];
#ifdef DEBUG
         debug ( "lvm_dir_cache -- calling scandir() with %s\n",
                 dirname);
#endif
         if ( ( dirent_count = scandir ( dirname, &dirent,
                                         lvm_dir_cache_select,
                                         alphasort)) > 0) {
            for ( n = 0; n < dirent_count; n++) {

               sprintf ( name, "%s/%s%c", dirname, dirent[n]->d_name, 0);
               if ( stat ( name, &stat_b) == -1) continue;
               if ( lvm_check_dev ( &stat_b, TRUE) == FALSE) continue;
         
               dir_cache_sav = dir_cache;
               if ( ( dir_cache =
                         realloc ( dir_cache,
                                   ( cache_size + 1) *
                                     sizeof ( dir_cache_t))) == NULL) {
                  fprintf ( stderr, "realloc error in %s [line %d]\n",
                                    __FILE__, __LINE__);
                  if ( dir_cache_sav != NULL) free ( dir_cache_sav);
                  dir_cache_sav = NULL;
                     return 0;
               }
         
               if ( ( dir_cache[cache_size].dev_name =
                         malloc ( strlen ( name) + 1 )) == NULL) {
                  fprintf ( stderr, "malloc error in %s [line %d]\n",
                                    __FILE__, __LINE__);
                  free ( dir_cache);
                  dir_cache = dir_cache_sav = NULL;
                  return 0;
               }
         
               if ( lvm_dir_cache_hit ( dir_cache, cache_size,
                                        stat_b.st_rdev) == FALSE) {
                  strcpy ( dir_cache[cache_size].dev_name, name);
                  dir_cache[cache_size].st_rdev = stat_b.st_rdev;
                  dir_cache[cache_size].st_mode = stat_b.st_mode;
                  cache_size++;
               }
            }
            for ( n = 0; n < dirent_count; n++) free ( dirent[n]);
            free ( dirent);
         }
#ifdef DEBUG
         debug ( "lvm_dir_cache -- AFTER calling scandir() "
                 "with %s\n", dirname);
#endif
      }
   }

   *dir_cache_ptr = dir_cache;
#ifdef DEBUG
         debug ( "lvm_dir_cache -- LEAVING wit cache_size: %d\n", cache_size);
#endif
   return cache_size;
}


/* internal selection function for scandir() */
/* gets all c*, sd*, hd* and md* with name adn st_rdev into cache */
int lvm_dir_cache_select ( struct dirent *dirent) {
   /* devfs scsiand ide /dev/{sd, ide/hd}/c#t#d#s# specials */
   if ( ( strcmp ( dirname, "/dev/sd") == 0 ||
          strcmp ( dirname, "/dev/ide/hd") == 0) &&
          dirent->d_name[0] == 'c' &&
          isalnum ( dirent->d_name[1]))
      return 1;

   /* devfs multiple devicees */
   if ( strcmp ( dirname, "/dev/md") == 0 &&
        isalnum ( dirent->d_name[0]))
      return 1;

   if ( strcmp ( dirname, "/dev") == 0 &&
        strlen ( dirent->d_name) > 2 &&
        ( dirent->d_name[0] == 's' ||    /* old /dev/sd[a-z]* special */
          dirent->d_name[0] == 'h' ||    /* old /dev/hd[a-z]* special */
          dirent->d_name[0] == 'm') &&   /* old /dev/md[a-z]* special */
          dirent->d_name[1] == 'd')      /* the 'd' of the 3 above */
      return 1;

   return 0;
}



int lvm_dir_cache_hit ( dir_cache_t* dir_cache, int cache_size, dev_t rdev) {
   int d = 0;

   for ( ; d < cache_size; d++) if ( dir_cache[d].st_rdev == rdev) return TRUE;
   return FALSE;
}
